﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

using Sample.Data;
using Sample.Data.BusinessObjects;
using Infragistics.Web.Mvc;

namespace Sample.Controllers
{
    public class ChainingController : Controller
    {
        public ActionResult DefinedColumns()
        {
            IQueryable<MyVirtualMachine> query = GetVirtualMachineData();
            return View(query);
        }

        [GridDataSourceAction]
        public ActionResult GetData()
        {
            return View(GetVirtualMachineData());
        }

        public ActionResult EditingSaveChanges()
        {
            DevelopmentDBEntities context = new DevelopmentDBEntities();

            IQueryable<VirtualMachines> virtualMachines = from v in context.VirtualMachines select v;

            ViewData["GenerateCompactJSONResponse"] = false;
            GridModel m = new GridModel();
            List<Transaction<MyVirtualMachine>> transactions = m.LoadTransactions<MyVirtualMachine>(HttpContext.Request.Form["ig_transactions"]);

            foreach (Transaction<MyVirtualMachine> t in transactions)
            {
                string rowId = t.rowId;
                switch (t.type)
                {
                    case "row":
                        var virtualMachine = (from v in virtualMachines
                                              where v.VirtualMachineID == rowId
                                              select v).Single();

                        if (t.row != null)
                        {
                            virtualMachine.DeveloperID = t.row.DeveloperId;
                        }

                        virtualMachine.OperatingSystem = t.row.OperatingSystem;

                        virtualMachine.DelegatedSpace = t.row.DelegatedSpace;

                        virtualMachine.ExpirationDate = t.row.ExpirationDate;
                        break;

                    case "newrow":
                        VirtualMachines newVM = new VirtualMachines()
                        {
                            VirtualMachineID = t.row.VirtualMachineId,
                            DeveloperID = t.row.DeveloperId,
                            OperatingSystem = t.row.OperatingSystem,
                            DelegatedSpace = t.row.DelegatedSpace,
                            ExpirationDate = t.row.ExpirationDate
                        };
                        context.VirtualMachines.AddObject(newVM);
                        // context.AddToProducts(newPr);
                        break;
                    case "deleterow":
                        var product1 = (from v in virtualMachines
                                        where v.VirtualMachineID == rowId
                                        select v).Single();
                        context.DeleteObject(product1);
                        break;
                }

                context.SaveChanges();
            }

            JsonResult result = new JsonResult();
            Dictionary<string, bool> response = new Dictionary<string, bool>();
            response.Add("Success", true);
            result.Data = response;
            return result;
        }

        private IQueryable<MyVirtualMachine> GetVirtualMachineData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();
            var query = from v in entities.VirtualMachines
                        orderby v.VirtualMachineID
                        select  new MyVirtualMachine
                            {
                                VirtualMachineId = v.VirtualMachineID,
                                DeveloperId = v.DeveloperID,
                                OperatingSystem = v.OperatingSystem,
                                DelegatedSpace = v.DelegatedSpace,
                                ExpirationDate = v.ExpirationDate
                            };

            return query;
        }

        private IQueryable<MyProject> GetProjectData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();
            var query = from p in entities.Projects
                        orderby p.ProjectID
                        select new MyProject
                        {
                            ProjectId = p.ProjectID,
                            Name = p.Name,
                            StartDate = p.StartDate,
                            EndDate = p.EndDate,
                            IsPostponed = p.IsPostponed,
                            Developers = from d in p.Developers
                                         select new MyDeveloper
                                         {
                                             DeveloperId = d.DeveloperID,
                                             ProjectId = d.ProjectID,
                                             Name = d.Name,
                                             Position = d.Position,
                                             HireDate = d.HireDate,
                                             Salary = d.Salary,
                                             VirtualMachines = from v in d.VirtualMachines
                                                               select new MyVirtualMachine
                                                               {
                                                                   VirtualMachineId = v.VirtualMachineID,
                                                                   DeveloperId = v.DeveloperID,
                                                                   OperatingSystem = v.OperatingSystem,
                                                                   DelegatedSpace = v.DelegatedSpace,
                                                                   ExpirationDate = v.ExpirationDate
                                                               }
                                         },
                            Scrums = from s in p.Scrums
                                     select new MyScrum
                                     {
                                         ProjectId = s.ProjectID,
                                         MeetingId = s.MeetingID,
                                         OccurredOn = s.OccurredOn,
                                         ScrumMaster = s.ScrumMaster,
                                         Notes = s.Notes,
                                         Priority = s.Priority,
                                         AreAllRequired = s.AreAllRequired,
                                         ScrumMeetings = new MyScrumMeeting()
                                         {
                                             MeetingId = s.ScrumMeetings.MeetingID,
                                             Name = s.ScrumMeetings.Name,
                                             Description = s.ScrumMeetings.Description,
                                             MaxDuration = s.ScrumMeetings.MaxDuration
                                         }
                                     },
                            Tasks = from t in p.Tasks
                                    select new MyTask
                                    {
                                        TaskId = t.TaskID,
                                        ProjectId = t.ProjectID,
                                        Name = t.Name,
                                        ParentTaskId = t.ParentTaskID,
                                        IsCompleted = t.IsCompleted,
                                        Checkins = from c in t.Checkins
                                                   select new MyCheckin
                                                   {
                                                       CheckinId = c.CheckinID,
                                                       TaskId = c.TaskID,
                                                       CheckinDate = c.CheckinDate,
                                                       Files = from f in c.Files
                                                               select new MyFile
                                                               {
                                                                   FileId = f.FileID,
                                                                   CheckinId = f.CheckinID,
                                                                   Name = f.Name,
                                                                   Size = f.Size
                                                               }
                                                   }
                                    }
                        };
            return query;
        }
    }
}
